@@ -1181,6 +1181,7 @@ struct sb_envp_ctx sb_new_envp(char **envp, bool insert)
 	found_var_cnt = 0;
 	memset(found_vars, 0, sizeof(found_vars));
 
+	/* Iterate through user's environment and check against expected. */
 	str_list_for_each_item(envp, entry, count) {
 		for (i = 0; i < num_vars; ++i) {
 			if (found_vars[i])
@@ -1192,6 +1193,14 @@ struct sb_envp_ctx sb_new_envp(char **envp, bool insert)
 		}
 	}
 
+	/* Treat unset and expected-unset variables as found. This will allow us
+	 * to keep existing environment. */
+	for (i = 0; i < num_vars; ++i) {
+		if (vars[i].value == NULL && found_vars[i] == NULL) {
+			++found_var_cnt;
+		}
+	}
+
 	/* Now specially handle merging of LD_PRELOAD */
 	char *ld_preload;
 	bool merge_ld_preload = found_vars[0] && !strstr(found_vars[0], sandbox_lib);
